'Filename   Encoder_100.bas
'Version    1.00
'Date       2012-03-19
'Purpose    Test of ALPS STEC11B13 encoder (reichelt.de)
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATmega8
'           Crystal 16 MHz
'           2x16 Text LCD @ PortC
'           Encoder A @ PortB.0
'           Encoder B @ PortB.1
'Compiler   BASCOM-AVR 2.0.7.3
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'           Flash used: 1420 bytes (17%)
'Edit history
'V1.00      2012-03-19, first release

'==============================================================================

'FUSE-Bits (Ponyprog-Settings, [ ]=no check, [x]=check)
'In ATmega data sheets: Bit = 1 -> not set = [ ] Ponyprog
'                       Bit = 0 -> set     = [x] Ponyprog
'- OCDEN    [ ]
'- JTAGEN:  [ ] (disabled!, otherwise LCD at PORT.C does not work)
'- EESAVE   [ ] Save EEProm on erase chip
'- BOOTSZ1  [x] Select Boot size
'- BOOTSZ0  [x] Select Boot size
'- BOOTRST  [ ] Select reset vector
'- Crystal oscillator
'  CKOPT:   [ ] (=1, low Oscillator amplitude or=1, [x], high ampl.)
'  CKSEL3:  [ ] (=1, Select Clock Source)
'  CKSEL2:  [ ] (=1, as well)
'  CKSEL1:  [ ] (=1, as well)
'  CKSEL0:  [ ] (=1, crystal osc. fast start up 16k CK + 4ms)
'  SUT1:    [ ] (=1, as well)
'  SUT0:    [x] (=0, as well)

$regfile = "m8def.dat"                            'ATmega8
$crystal = 16000000                               '16 MHz xtal
$hwstack = 40
$swstack = 40
$framesize = 40

'Configure LCD on Port C ------------------------------------------------------
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db7 = PortC.0 , Db6 = PortC.1 , Db5 = PortC.2
Config Lcdpin = Pin , Db4 = PortC.3 , E = PortC.4 , Rs = PortC.5
Config Lcdbus = 4

'Configure Encoder on Port B --------------------------------------------------
DDRB = &B00000000                                 'PortB = input
PORTB = &B11111111                                'Activate Pullup
'                                                 'PINB.0 = Encoder Pin A
'                                                 'PINB.1 = Encoder Pin B
'                                                 'PINB.2 = Encoder key
Enc_A Alias PINB.0                                'Encoder A
Enc_B Alias PINB.1                                'Encoder B
Enc_K Alias PINB.2                                'Encoder key

'Configure Timer0 interrupt for encoder  --------------------------------------
'Timer0 = 8 bit
'Overflow time: overflow counts * Prescale / crystal frequency
'= 256 * 256 / 16.000.000 Hz = 4,1 msec = ~ 244 Hz
'Here Prescale = 256
'The encoder is polled every 4,1 msec. To shorten time set PresetTimer 0 > 0
'Possible Prescales: 8, 64, 256, 1024
Config Timer0 = Timer , Prescale = 256
Const PresetTimer0 = 0                            'Initialize Timer0
On Timer0 Timer0_isr                              'Timer0 Interrupt service routine

'Common Variables, multiply used ----------------------------------------------
Dim lngFreq As Long
Dim lngStep As Long
Dim strFreq As String * 9

'Variables Encoder  -----------------------------------------------------------
Dim bytIncr As Byte                               '=2: Encoder incr., =1: decr., =0: ./.
Dim bytEnc_old As Byte
Dim bytEnc_new As Byte
Dim bitKey_Enc As Bit                             'Encoder key, =1: pressed
Const DoubleStep = 0                              '=1: intermediate position detected

Declare Sub ClearLCDLine(byVal bytLine As Byte)

'Main program  ****************************************************************
Cls
Cursor Off

lngFreq = 14000000
lngStep = 10
Locate 1 , 1
strFreq = Str(lngFreq)
strFreq = Format(strFreq , " 0.000000")
LCD strFreq

bytEnc_old = 0                                    'Encoder data
bytEnc_new = 0                                    'Encoder data
bytIncr = 0

Enable Timer0                                     'Interrupt encoder
Enable Interrupts                                 'enable all Interrupts

Do

   Disable timer0                                 'Stop Timer0 for display

   Select Case bytIncr                            'Set/show TRX freq.
      Case 2                                      'Encoder incremented
         lngFreq = lngFreq + lngStep
      Case 1                                      'Encoder decremented
         lngFreq = lngFreq - lngStep
   End Select

   If bytIncr > 0 Then                            'Freq. was changed
      strFreq = Str(lngFreq)
      strFreq = Format(strFreq , " 0.000000")
      Call ClearLCDLine(2)
      LCD strFreq
      bytIncr = 0                                 'Reset increment flag
   End If

   Enable Timer0                                  'Restart Timer0

Loop

End

'End of main program **********************************************************

'------------------------------------------------------------------------------
Timer0_isr:
'Encoder interrupt on Timer0 overflow
'While data are displayed Timer0 should be stopped
'Output: bytIncr = 2: increment, =1: decrement if encoder was changed
'Having processed the bytIncr data in the main program reset bytIncr = 0
'Switching pattern of ALPS STEC11 encoder:
'Turning clockwise              ... counter clockwise
'A    B                             A     B
'1    1  start detent position      1     1
'1    0  intermediate position 1    0     1
'0    0  intermediate position 2    0     0
'0    1  intermediate position 3    1     0
'1    1  next detent position       1     1

   Timer0 = PresetTimer0
   bytEnc_new.0 = Enc_A                           'Set bit 0 of bytEnc_new
   bytEnc_new.1 = Enc_B                           'Set bit 1 of bytEnc_new

   'Check encoder pattern if encoder is changed
   'bytIncr = 1: decremented (turned counter clockwise)
   'bytIncr = 2: incremented (turned         clockwise)
   If bytEnc_new <> bytEnc_old Then               'encoder is changed
      'from intermediate position 3 to quiescent position
      If bytEnc_new = &B00000011 And bytEnc_old = &B00000010 Then bytIncr = 2
      If bytEnc_new = &B00000011 And bytEnc_old = &B00000001 Then bytIncr = 1
      'from intermediate position 1 to intermediate position 2
      'DoubleStep = 1:  intermediate position 2 gives an additional tick
      '                 2 ticks between 2 detent positions on slow rotation
      '           = 0:  1 tick  between 2 detent positions
      #If DoubleStep = 1
         If bytEnc_new = &B00000000 And bytEnc_old = &B00000001 Then bytIncr = 2
         If bytEnc_new = &B00000000 And bytEnc_old = &B00000010 Then bytIncr = 1
      #EndIf
      bytEnc_old = bytEnc_new                     'old <- new for the next time
   End If
Return

'------------------------------------------------------------------------------
Sub ClearLCDLine(byVal bytLine As Byte)
'Clear LCD line bytLine
   Locate bytline , 1
   LCD SPC(16)
   Locate bytline , 1
End Sub
'------------------------------------------------------------------------------